import { content, formElements, quizElements } from "../../fixtures/data"

/**
 * Creates content with given name and of given type content/form
 *
 * @param {string} contentName name of the content
 * @param {string} contentType type of the content - possible options 'form'/'content'/'document'
 * @param {string} docType type of the content - possible options 'Document for signing'/'Document for inventory'
 */
 export function createContent(contentName, contentType, docType='') {

    if (contentType === 'form') {
        cy.get('button:contains("Add")').first().click()

    } else if (contentType === 'content') {
        cy.get('button:contains("Add")').eq(1).click()

    } else if (contentType === 'document') {
        cy.get('button:contains("Add")').last().click()
        cy.selectOption('Type', docType)

    }

    cy.get('div[id^=headlessui-dialog-panel]').should('be.visible')

    // fills info
    cy.get('[name="name"]').should('be.visible').type(contentName)
    cy.selectOption('Language', 'en')

    // saves
    cy.get('[type="submit"]').click()
    cy.url().should('contain', 'content-detail')
    cy.get('#mainContent > div.login-spinner').should('not.exist')

    // checks if created
    cy.visit('/content')
    cy.get('#mainContent > div.login-spinner').should('not.exist')
    cy.get('body').should('contain', contentName)
}

/**
 * Opens a content of a given name
 *
 * @param {string} contentName name of the content
 */
export function openContent(contentName) {

   cy.wait(1000)

   cy.get('table').should('contain', contentName).then(() => {
      cy.get('td:contains("' + contentName + '")').click().then(() => {
         cy.url().should('contain', '/content-detail')
      })
   })
} 

/**
 * Deletes content of a given name
 *
 * @export
 * @param {string} contentName name of the content
 * @param {string} contentType type of the content - possible options 'form'/'content'/'document' 
 */
export function deleteContent(contentName, contentType) {

    let t = ''

    switch (contentType) {
        case 'form':
            t = 'first'
            break
        case 'content':
            t = 'eq(1)'
            break
        case 'document':
            t = 'last'
            break
    }

    cy.delete('Delete')

    // checks
    cy.url().should('not.contain', 'detail')
    cy.get('#mainContent > div.login-spinner').should('not.exist')
    cy.get('table:' + t + ' > tbody').should('exist').and('not.contain', contentName)
}

/**
 * Checks "Public template for everybody to see" to keep content in Wiki for employee to see
 */
export function keepInWiki() {

    cy.get('label:contains("Public template for everybody to see")').within(() => {
        // cy.get('span:contains("Public template for everybody to see")')
        cy.get('input').click()
    })

    cy.get('label:contains("Public template for everybody to see") input').should('be.checked')
}

/**
 * Creates a segment of a given name
 *
 * @param {string} segmentName name of the segment
 * @param {string} segmentType type of the segment - possible options 'Form'/'Upload files'/'Content'/'Quiz'/'List of files'/'Video'/'Notion'
 */
export function createSegment(segmentName, segmentType) {

    cy.get('button:contains("Add new segment")').click()

    cy.get('div[id^=headlessui-dialog-panel]').should('be.visible')
 
    // fills info
    cy.get('[name="segmentName"]').type(segmentName)
    cy.selectOption('Segment type', segmentType)

    // saves
    cy.get('[type="submit"]').click()

    // checks
    cy.get('div[id^=headlessui-tabs-panel]:last').should('contain', segmentName)
}

/**
 * Deletes segment of given name in given content
 *
 * @export
 * @param {string} segmentName name of the segment
 */
export function deleteSegment(segmentName) {

    cy.contains(segmentName).parent().next().click()

    cy.get('[role="button"]:contains("Delete")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
    cy.get('button:contains("Confirm")').click()

    // check if deleted
    cy.get('div[id^=headlessui-tabs-panel]').should('not.contain', segmentName)
}

/**
 * Opens preview
 */
export function openPrev() {

    cy.get('button:contains("Preview")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
}

/**
 * Closes preview
 */
export function closePrev() {

    cy.get('body').type('{esc}', { force: true })
    cy.get('div[id^="headlessui-dialog-panel"]').should('not.exist')
}

// FORM ↓↓↓ ---------------------------------------------------------------

/**
 * Adds an element in a form 
 *
 * @param {object} element object with element information
 */
export function addFormElement(element) {

    cy.get('button:contains("add block")').click()
    cy.get('li[data-rbd-draggable-id^="item"]:last').should('exist').and('be.visible')

    // fill info in
    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Question") + ').clear().type(element.question)
    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Description") + ').clear().type(element.description)

    selectOptionInForms('Type', element.type)
    selectOptionInForms('Width', element.width)

    if (element.required) {
        cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Required")').click()

    }

    // additional info
    if (element.type === 'Dropdown' || element.type === 'Checkboxes') {
        cy.get('input[placeholder="Enter option"]:last').type('choice_1')
        cy.get('button:contains("Add option"):last').click()
        cy.get('input[placeholder="Enter option"]:last').type('choice_2')

    } else if (element.type === 'Linear scale') {
        cy.get('input[placeholder="Great"]').type('Best')
        cy.get('input[placeholder="Bad"]').type('Worst')

    } else if (element.type === 'Date') {
        cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Date") + ').type('01.01.2024')

    } else if (element.type === 'Time') {
        selectOptionInForms('Hours', '10')
        selectOptionInForms('Minutes', '58')

    }

    // save
    cy.get('button:contains("Save")').click()
    cy.get('.Toastify > div').should('exist')
}

/**
 * Help function for selecting options in form elements
 * 
 * @param {string} labelName name of the label
 * @param {string} optionName name of the option to be selected
 */
function selectOptionInForms(labelName, optionName) {

    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("' + labelName + '"):last + ').click()
    cy.get('.selectTw__menu-list').should('be.visible')
    cy.get('.selectTw__option:contains("' + optionName + '")').should('exist').click()
    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("' + labelName + '"):last ~ div, .selectTW__single-value').should('contain', optionName)
}

/**
 * Creates all form elements
 */
export function createAllFormElements() {

    // create elements
    formElements.forEach(element => {
        addFormElement(element)

    })

    // save
    cy.get('button:contains("Save")').click()

    // check after reload
    cy.reload()
    cy.get('ul > li').should('have.length', 7)
}

/**
 * Checks preview 'Form' containing all elements
 */
export function previewFormAll() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
        .and('contain', 'test_short_text')
        .and('contain', 'test_paragraph')
        .and('contain', 'test_dropdown')
        .and('contain', 'test_scale')
        .and('contain', 'test_evaluation')
        .and('contain', 'test_checkbox')
        .and('contain', 'test_date')
        // .and('contain', 'test_time')
}

/**
 * Creates form elements for custom fields
 */
export function createCustomForm() {

    //create elements
    addFormElement({ question: 'How well did he/she fit in the team?', description: 'team fit', type: 'Short answer', width: '50', required: false })
    addFormElement({ question: 'How well are they performing?', description: 'performance', type: 'Short answer', width: '50', required: false })

    //save
    cy.get('button:contains("Save")').click()

    //check after reload
    cy.reload()
    cy.get('ul > li').should('have.length', 2)
}

/**
 * Checks preview custom 'Form'
 */
export function previewFormCustom() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
        .and('contain', 'How well did he/she fit in the team?')
        .and('contain', 'How well are they performing?')
}

// UPLOAD ↓↓↓ ---------------------------------------------------------------

/**
 * Uploads files to a form 
 */
export function uploadFilesForm() {

    cy.get('div[id*="headlessui-dialog-panel"]:contains("Upload files")')
    .attachFile(['jpg.jpg', 'excel.xlsx'], { subjectType: 'drag-n-drop' })
}

/**
 * Checks preview 'Upload file'
 */
export function previewUpload() {

    cy.get('div[id*="headlessui-dialog-panel"]').should('contain', 'Upload files')
}

// CONTENT ↓↓↓ ---------------------------------------------------------------

/**
 * Fills segment type 'Content' with long text trying all styles
 */
export function fillContentLong(buttonText='Save') {

    cy.get('.ql-editor').clear()

    //fills content with text
    cy.get('.ql-editor').type('I am a link to another site.{enter}{enter}').type('{selectall}')
    cy.get('.ql-link > svg').click()

    cy.get('.ql-tooltip > input').clear().type('google.com')
    cy.get('.ql-action').click()

    cy.get('.ql-editor').click()

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(0)').should('be.visible').click({force: true}) //heading1
    cy.get('.ql-editor').type('Heading1.{enter}')

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(1)').click() //heading2
    cy.get('.ql-editor').type('Heading2.{enter}')

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(2)').click() //heading3
    cy.get('.ql-editor').type('Heading3.{enter}')

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(3)').click() //normal

    cy.get('.ql-editor').type('I am a normal text.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-editor').type('And I am a bold text.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-italic > svg').click()
    cy.get('.ql-editor').type('Hihi, I am italic.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-editor').type('Hah, jokes on you, I am both italic and bold.{enter}')
    cy.get('.ql-underline > svg').click()
    cy.get('.ql-editor').type('I am even underlined so I am EXTRA important, muhahah.{enter}{enter}')

    cy.get('.ql-bold > svg').click()
    cy.get('.ql-italic > svg').click()
    cy.get('.ql-underline > svg').click()

    cy.get('[value="ordered"] > svg').click()
    cy.get('.ql-editor').type('Item no. 0 in an ordered list.{enter}')
    .type('Item no. 1 in an ordered list.{enter}')
    .type('Item no. 2 in an ordered list.{enter}')
    .type('Item no. 3 in an ordered list.{enter}{enter}{enter}').tab()

    cy.get('[value="bullet"] > svg').click()
    cy.get('.ql-editor').type('Item no. 1 in an unordered list.{enter}')
    .type('Item no. 2 in an unordered list.{enter}').tab()
    .type('Item no. 2.1 in an unordered list.{enter}{enter}{enter}')

    cy.get('.ql-clean > svg').click()

    cy.get('.ql-editor').type('I am a normal text.{enter}')

    //save the text
    cy.get('button:contains("' + buttonText + '")').click()
}

/**
 * Checks if preview contains the text filled in text editor - long text
 */
export function previewContentLong() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
        .and('contain', 'I am a link to another site.')
        .and('contain', 'Heading1.')
        .and('contain', 'Heading2.')
        .and('contain', 'Heading3.')
        .and('contain', 'I am a normal text.')
        .and('contain', 'And I am a bold text.')
        .and('contain', 'Hihi, I am italic.')
        .and('contain', 'Hah, jokes on you, I am both italic and bold.')
        .and('contain', 'I am even underlined so I am EXTRA important, muhahah.')
        .and('contain', 'Item no. 0 in an ordered list.')
        .and('contain', 'Item no. 1 in an ordered list.')
        .and('contain', 'Item no. 2 in an ordered list.')
        .and('contain', 'Item no. 3 in an ordered list.')
        .and('contain', 'Item no. 1 in an unordered list.')
        .and('contain', 'Item no. 2 in an unordered list.')
        .and('contain', 'Item no. 2.1 in an unordered list.')
        .and('contain', 'I am a normal text.')
}

/**
 * Fills segment type 'Content' with short text 
 */
export function fillContentShort(buttonText='Save') {

    //fills content with text
    cy.get('.ql-editor').type('Some funky text text.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-editor').type('That is very important.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-italic > svg').click()
    cy.get('.ql-editor').type('And kind of weird.{enter}')
    cy.get('.ql-bold > svg').click()
    cy.get('.ql-editor').type('Funkyyy.{enter}')
   
    //save the text
    cy.get('button:contains("' + buttonText + '")').click()
}

/**
 * Checks if preview contains the text filled in text editor - short text
 */
export function previewContentShort() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
        .and('contain', 'Some funky text text.')
        .and('contain', 'That is very important.')
        .and('contain', 'And kind of weird.')
        .and('contain', 'Funkyyy.')
}

//  QUIZ ↓↓↓ ---------------------------------------------------------------

/**
 * Adds an element in a quiz 
 *
 * @param {object} element object with element information
 */
export function addQuizElement(element) {

    cy.get('button:contains("add block")').click()
    cy.get('li[data-rbd-draggable-id^="item"]:last').should('exist').and('be.visible')

    // fill info in
    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Question") + ').clear().type(element.question)
    cy.get('li[data-rbd-draggable-id^="item"]:last').find('label:contains("Description") + ').clear().type(element.description)

    selectOptionInForms('Type', element.type)

    // additional info
    if (element.type === 'Multiple choice') {
        cy.get('input[value="10"]:last').clear().type('choice_1')
        cy.get('input[value="choice_1"]:last').siblings('div').find('input').click({ force: true})
        cy.get('input[value="choice_1"]:last').siblings('div').find('input').should('not.be.checked')
        cy.get('input[value="20"]:last').clear().type('choice_2')
        cy.get('button:contains("Add option"):last').click()
        cy.get('input[placeholder="Add option"]:last').clear().type('choice_3')

    } else if (element.type === 'Single choice') {
        cy.get('input[value="10"]:last').clear().type('single_1')
        cy.get('input[value="single_1"]:last').siblings('div').find('input').click({ force: true})
        cy.get('input[value="single_1"]:last').siblings('div').find('input').should('be.checked')
        cy.get('input[value="20"]:last').clear().type('single_2')
        cy.get('button:contains("Add option"):last').click()
        cy.get('input[placeholder="Add option"]:last').clear().type('single_3')

    }

    // save
    cy.get('button:contains("Save")').click()
    cy.get('.Toastify > div').should('exist')
}

/**
 * Creates all quiz elements
 */
export function createQuizElements() {

    quizElements.forEach(element => {
        addQuizElement(element)

    })

    // save
    cy.get('button:contains("Save")').click()

    // check after reload
    cy.reload()
    cy.get('ul > li').should('have.length', 2)
}

//  FILES ↓↓↓ ---------------------------------------------------------------

/**
 *  Uploads files to the 'List of files'
 * 
 * @param {[string]} files list of files to be uploaded
 */
export function addFiles(files) {

    cy.get('input[type="file"]')
    .attachFile(files, { subjectType: 'drag-n-drop' })

    // checks if files are uploaded
    files.forEach(file => {
        cy.get('.grid').should('contain', file)

    })
}

/**
 * Checks preview 'List of files'
 * 
 * @param {[string]} files list of files that have been uploaded
 */
export function previewFiles(files) {

    files.forEach(file => {
        cy.get('div[id*="headlessui-dialog-panel"]')
            .should('be.visible')
            .and('contain', file)

    })
}

/**
 * Delete files from the 'List of files'
 * 
 * @param {[string]} files list of files to be deleted
 */
export function deleteFiles(files) {

    files.forEach(file => {
        cy.get('a:contains("' + file + '")').parent().parent().find('button').click()
        cy.get('li:contains("Delete file")').click()
        cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
        cy.get('button:contains("Confirm")').click()

        cy.get('.grid').should('not.contain', file)
    })
}

//  VIDEO ↓↓↓ ---------------------------------------------------------------

/**
 *  Adds youtube video to 'Video'
 * 
 * @param {string} videoID ID of the youtube video
 */
export function addYoutube(videoID) {

    cy.selectOption('Video type', 'Youtube')
    cy.get('label:contains("Video ID") + ').type(videoID)

    // saves
    cy.get('[type="submit"]').click()
    cy.get('.Toastify > div').should('exist')
}

/**
 * Checks preview 'Video' for Youtube video
 * 
 * @param {string} videoID ID of the youtube video
 */
export function previewVideoYoutube(videoID) {

    cy.wait(2500)
    // cy.get('div[style*="' + videoID + '"]').should('exist')

    cy.get('#widget2:contains("' + videoID + '")').should('exist')
}

//  NOTION ↓↓↓ ---------------------------------------------------------------


//  SIGNATURE ↓↓↓ ---------------------------------------------------------------

/**
 * Fills segment type 'Content' with text (contract)
 */
export function fillContentDoc() {

    //fills content with text
    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(0)').should('be.visible').click({force: true}) //heading1
    cy.get('.ql-editor').type('Employment contract agreement{enter}')

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(1)').click() //heading2
    cy.get('.ql-editor').type('Parties{enter}')

    cy.get('.ql-picker-label:first').click()
    cy.get('.ql-picker-options > span:eq(3)').click() //normal

    cy.get('.ql-editor').type('This employment contract agreement is between ')
    cy.get('div:contains("firstname"):last').click()
    cy.get('.ql-editor').type(' ')
    cy.get('div:contains("surname"):last').click()
    cy.get('.ql-editor').type(', {enter}with an address of ')
    cy.get('div:contains("address"):last').click()
    cy.get('.ql-editor').type(', ')
    cy.get('div:contains("psc"):last').click()
    cy.get('.ql-editor').type(' ')
    cy.get('div:contains("city"):last').click()
    cy.get('.ql-editor').type(', ')
    cy.get('div:contains("country"):last').click()
    cy.get('.ql-editor').type('{enter}and{enter}Hai Yen, {enter}with an address of Sychrova, 59101 Zdar, Ceska Republika')

    //save the text
    cy.get('button:contains("Save")').click()
}

/**
 * Checks if preview contains the text filled in text editor - document
 */
export function previewContentDoc() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
        .and('contain', 'Employment contract agreement')
        .and('contain', 'Parties')
        .and('contain', 'This employment contract agreement is between')
        .and('contain', 'Verification code')
}

//  INVENTORY ↓↓↓ ---------------------------------------------------------------

/**
 * Checks preview 'Inventory'
 */
export function previewInventory() {

    cy.get('#headlessui-portal-root > div > div > div > div > .ml-auto')
        .should('be.visible')
}

//  WIKI ↓↓↓ ---------------------------------------------------------------

/**
 * Checks wiki from an employee's account
 * 
 * @param contents list of contents name that are supposed to be visible in wiki on employee's account
 */
export function checkWiki(contents) {

    cy.get('tbody tr').should('have.length', contents.length)

    contents.forEach(content => {
        cy.get('tr td').should('contain', content)
    })
}
