import { fillPositionInfo } from '../../../support/testFunctions/employee/position'

/**
 * Adds a new employee manually by creating a new person
 * 
 * @param {object} person personal information of a person we are creating
 * @param {object} position information about position that the person is being assigned to
 * @param {string} mode if the position is being picked from existing ones or is being created, has two options 'pick' and 'create'
 * @param {boolean} template if the position will use template then 'true', if it's going to be skipped then 'false'
 */
export function addEmployee(person, position, mode='pick', template) {

    cy.get('button:contains("Add")').click()
    cy.get('div:contains("Manually"):last').click()
    cy.url().should('contain', '/add')

    // fill in personal information
    fillPersonalInfo(person)

    cy.get('button:contains("Save employee")').click()

    if (position === null) {
        cy.visit('/employees')
        cy.get('#mainContent > div.login-spinner').should('not.exist')

    } else {

        // fill in position information
        fillPositionInfo(position, mode, template)

        cy.get('button:contains("Create position")').click()

        // check
        cy.get('#mainContent > div.login-spinner').should('not.exist')
        cy.get('form').should('contain', position.name)

    }
}

/**
 * Deletes employee of given name
 * 
 * @param {string} surname surname of the person to be deleted
 */
export function deleteEmployee(surname) {

    cy.delete('Delete')

    cy.url().should('contain', 'onbee.link/employees')
    cy.get('table').should('not.contain', surname)
}

/**
 * Fills information of a new person in a 'Add' employee form.
 *  
 * @param {object} person personal information of a person we are creating
 */
export function fillPersonalInfo(person) {

    cy.get('[name="firstname"]').type(person.name)
    cy.get('[name="surname"]').type(person.surname)
    cy.get('[name="middlename"]').type(person.middlename)
    cy.selectOption('Gender', person.sex)
    cy.get('label:contains("Date of birth") + div').type(person.birthday + '{enter}')

    cy.get('[name="address"]').type(person.address)
    cy.get('[name="city"]').type(person.city)
    cy.get('[name="psc"]').type(person.psc)
    cy.get('[name="country"]').type(person.country)

    cy.get('[name="privateEmail"]').type(person.privateEmail)
    cy.get('[name="workEmail"]').type(person.workEmail)
    cy.get('label:contains("Private phone number") + div').clear().type(person.privPhone)
    cy.get('label:contains("Work phone number") + div').clear().type(person.workPhone)

    cy.get('[name="emergencyContact"]').type(person.emergencyContact)
    cy.get('label:contains("Emergency phone number") + div').clear().type(person.emergPhone)

    cy.selectOption('Medical information', person.medInfo)

    cy.get('[name="doctorContact"]').type(person.docContact)
    cy.get(`label:contains("Doctor's phone number") + div`).clear().type(person.docPhone)
    cy.get('[name="importantMedicalInformations"]').type(person.importantMedInfo)
}

/**
 * Opens employees details using their surname
 *  
 * @param {string} surname surname of the person we want to see details of
 */
export function openEmployeeCard(surname) {

    cy.get('td:contains("' + surname + '")').click()
    cy.url().should('contain', '/personal-information/')
    cy.get('#mainContent > div.login-spinner').should('not.exist')
}

/**
 * Changes personal information
 * 
 * @param {object} person object containing personal information
 */
export function changePersonalInfo(person) {

    cy.get('[name="firstname"]').clear().type(person.name)
    cy.get('[name="middlename"]').clear().type(person.middlename)
    cy.get('label:contains("Gender") + div').type(person.sex + '{enter}')
    cy.get('label:contains("Date of birth") + div').clear().type(person.birthday + '{enter}')

    cy.get('[name="address"]').clear().type(person.address)
    cy.get('[name="city"]').clear().type(person.city)
    cy.get('[name="psc"]').clear().type(person.psc)
    cy.get('[name="country"]').clear().type(person.country)

    cy.get('[name="privateEmail"]').clear().type(person.privateEmail)
    cy.get('[name="workEmail"]').clear().type(person.workEmail)
    cy.get('label:contains("Private phone number") + div').clear().type(person.privPhone)
    cy.get('label:contains("Work phone number") + div').clear().type(person.workPhone)

    cy.get('[name="emergencyContact"]').clear().type(person.emergencyContact)
    cy.get('label:contains("Emergency phone number") + div').clear().type(person.emergPhone)

    cy.get('[name="bankAccount"]').clear().type(person.bankAccountNumber)
    cy.get('[name="bankAccountCode"]').clear().type(person.bankAccountCode)
    cy.get('[name="bankAccountSwift"]').clear().type(person.bankSwift)
    cy.get('[name="bankAccountBic"]').clear().type(person.bankBic)

    cy.get('[name="companyName"]').clear().type(person.companyName)
    cy.get('[name="ico"]').clear().type(person.ico)
    cy.get('[name="dic"]').clear().type(person.dic)
    cy.get('[name="contractorAddress"]').clear().type(person.companyAdress)
    cy.get('[name="contractorCity"]').clear().type(person.companyCity)
    cy.get('[name="contractorPsc"]').clear().type(person.companyPsc)
    cy.get('[name="contractorCountry"]').clear().type(person.companyCountry)

    cy.get('[name="doctorContact"]').clear().type(person.docContact)
    cy.get(`label:contains("Doctor's phone number") + div`).clear().type(person.docPhone)
    cy.get('[name="importantMedicalInformations"]').clear().type(person.importantMedInfo)

    cy.get('form:first button:contains("Save")').click({ force: true })
    cy.get('.Toastify > div').should('exist')
}

/**
 * Checks changed personal information
 * 
 * @param {object} person object containing personal information
 */
export function checkChangedPersonalInfo(person) {

    cy.get('[name="firstname"]').should('have.value', person.name)
    cy.get('[name="middlename"]').should('have.value', person.middlename)
    cy.get('label:contains("Gender") + div').should('contain', person.sexEnglish)
    cy.get('label:contains("Date of birth") + div input').should('have.value', person.birthday)

    cy.get('[name="address"]').should('have.value', person.address)
    cy.get('[name="city"]').should('have.value', person.city)
    cy.get('[name="psc"]').should('have.value', person.psc)
    cy.get('[name="country"]').should('have.value', person.country)

    cy.get('[name="privateEmail"]').should('have.value', person.privateEmail)
    cy.get('[name="workEmail"]').should('have.value', person.workEmail)
    cy.get('label:contains("Private phone number") + div input').should('have.value', person.privPhone)
    cy.get('label:contains("Work phone number") + div input').should('have.value', person.workPhone)

    cy.get('[name="emergencyContact"]').should('have.value', person.emergencyContact)
    cy.get('label:contains("Emergency phone number") + div input').should('have.value', person.emergPhone)

    cy.get('[name="bankAccount"]').should('have.value', person.bankAccountNumber)
    cy.get('[name="bankAccountCode"]').should('have.value', person.bankAccountCode)
    cy.get('[name="bankAccountSwift"]').should('have.value', person.bankSwift)
    cy.get('[name="bankAccountBic"]').should('have.value', person.bankBic)


    cy.get('[name="companyName"]').should('have.value', person.companyName)
    cy.get('[name="ico"]').should('have.value', person.ico)
    cy.get('[name="dic"]').should('have.value', person.dic)
    cy.get('[name="contractorAddress"]').should('have.value', person.companyAdress)
    cy.get('[name="contractorCity"]').should('have.value', person.companyCity)
    cy.get('[name="contractorPsc"]').should('have.value', person.companyPsc)
    cy.get('[name="contractorCountry"]').should('have.value', person.companyCountry)

    cy.get('[name="doctorContact"]').should('have.value', person.docContact)
    cy.get(`label:contains("Doctor's phone number") + div input`).should('have.value', person.docPhone)
    cy.get('[name="importantMedicalInformations"]').should('have.value', person.importantMedInfo)
}


/**
 * Opens tab in the employee's card of a given name
 * 
 * @param {string} tabName name of the tab - possible options 'Personal information'/'Career'/'Login'/
 * 'Outputs'/'Inventory'/'Internal notes/'Internal feedback'/'Assigned tasks'/'Events'
 */
export function openTab(tabName) {

    cy.get('span:contains("'+tabName+'")').click()

    // checks if opened
    let url = '/personal-information'
    
    switch (tabName) {
        case 'Career':
            url = '/career'
            break
        case 'Login':
            url = '/login'
            break
        case 'Outputs':
            url = '/outputs'
            break
        case 'Inventory':
            url = '/inventory'
            break
        case 'Internal notes':
            url = '/internal-notes'
            break
        case 'Internal feedback':
            url = '/internal-feedback'
            break
        case 'Team tasks':
            url = '/assigned-tasks'
            break
        case 'Events':
            url = '/events'
            break
        case 'Language':
        case 'Jazyk':
            url = '/profile/language'
            break
        case 'Change password':
            url = '/profile/password'
            break
    }

    cy.url().should('contain', url)
    cy.get('#mainContent > div.login-spinner').should('not.exist')
}

/**
 * Checks tab that are visible on employee's card based on the fact if the person is assigned to a position or not
 * 
 * @param {boolean} isPosition indicates if employee is assigned to a position
 */
export function checkTabs(isPosition) {
    
    if (!isPosition) {
        cy.get('.PriorityNav_Main:last').should('contain', 'Personal information')
            .and('not.contain', 'Career')
            .and('contain', 'Login')
            .and('not.contain', 'Outputs')
            .and('contain', 'Inventory')
            .and('not.contain', 'Internal notes')
            .and('not.contain', 'Internal feedback')
            .and('contain', 'Assigned tasks')
            .and('not.contain', 'Events')

    } else {
        cy.get('.PriorityNav_Main:last').should('contain', 'Personal information')
            .and('contain', 'Career')
            .and('contain', 'Login')
            .and('contain', 'Outputs')
            .and('contain', 'Inventory')
            .and('contain', 'Internal notes')
            .and('contain', 'Internal feedback')
            .and('contain', 'Assigned tasks')
            .and('contain', 'Events')

    }
}

// CAREER ↓↓↓ ---------------------------------------------------------------

/**
 * Adds date to the milestone
 * 
 * @param {string} milestoneName name of the milestone
 * @param {string} date date
 */
export function addDateToModule(milestoneName, date) {

    cy.get('h4:contains("'+milestoneName+'") + ').click()
    cy.get('li > div:contains("Edit")').click()

    cy.get('div[id^=headlessui-dialog-panel]').should('be.visible')

    cy.get('label:contains("Date") + ').type(date + '{enter}')

    cy.get('[type="submit"]').click()
}

// LOGIN ↓↓↓ ---------------------------------------------------------------

/**
 * Checks if employee's login tab contains linked user account
 * 
 * @param {object} user object containing info about user account
 */
export function checkLogin(user) {

    cy.get('div.grid:last').children().should('contain', user.email)
}

/**
 * Deletes employee's linked login
 * 
 * @param {object} user object containing info about user account
 */
export function deleteLogin(user) {

    cy.get('div.grid:last>:contains("' + user.email + '")>:first>:first>').click()
    cy.get('li> div:contains("Delete")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
    cy.get('button:contains("Confirm")').click()

    cy.get('div.grid:last').should('not.contain', user.email)
}

// OUTPUTS ↓↓↓ ---------------------------------------------------------------

/**
 * Checks if provided outputs are present in employee's output tab
 * 
 * @param {[object]} outputs list of objects containing info about each output (name and type)
 */
export function checkOutputs(outputs) {

    outputs.forEach(output => {
        cy.get('h3').should('contain', output.name)
            .and('contain', 'completed ' + output.type)
    })
}

// INTERNAL NOTES ↓↓↓ ---------------------------------------------------------------

/**
 * Checks internal notes that were filled with fillContentShort()
 */
export function checkInternalNotes() {

    cy.get('span:contains("Feuermann")').parent().parent().parent().within(() => {
        cy.get('p').should('contain', 'Some funky text text.')
            .and('contain', 'That is very important.')
            .and('contain', 'And kind of weird.')
            .and('contain', 'Funkyyy.')
    })
}

// INTERNAL FEEDBACK ↓↓↓ ---------------------------------------------------------------

/**
 * Fills in a internal feedback form, saves it and checks if it was saved properly
 * 
 * @param {object} answers object containing aswers to internal feedback 
 */
export function fillInInternalFeedback(answers) {

    cy.wait(2000)

    cy.get('label:contains("How well did he/she fit in the team?"):first + ').type(answers.a1)
    cy.get('label:contains("How well are they performing?"):first + ').type(answers.a2)

    cy.get('button[type="submit"]:contains("Save")').click()

    cy.wait(2000)

    // checks
    cy.get('i:first').should('contain', 'Feuermann')
    cy.get('i:first').should('contain', 'Feuermann').parent().parent().within(() => {
        cy.get('ul > li > div > label:contains("How well did he/she fit in the team?") + >>> input').should('have.value', answers.a1)
        cy.get('ul > li > div > label:contains("ow well are they performing?") +  >>> input').should('have.value', answers.a2)
    })
}

// TEAM TASKS ↓↓↓ ---------------------------------------------------------------

/**
 * Checks if the tab 'Assigned tasks' contains tasks of given names
 * 
 * @param {[string]} tasks list of task names expected to be there
 */
export function checkAssignedTasks(tasks) {
    
    tasks.forEach(task => {
        cy.get('div >> div[role="button"]').should('have.length', tasks.length)
        cy.get('div >> div[role="button"]').should('contain', task)
    })
}

// LANGUAGE ↓↓↓ ---------------------------------------------------------------

/**
 * Changes language of the system
 * 
 * @param {string} language language to change into - options 'en'/'cs'
 */
export function changeLanguage(language) {

    if (language === 'cs') {
        cy.selectOption('Language', language)
        cy.get('[type="submit"]').click()
        cy.get('h1').should('contain', 'Osobní nastavení')


    } else if (language === 'en') {
        cy.selectOption('Jazyk', language)
        cy.get('[type="submit"]').click()
        cy.get('h1').should('contain', 'Personal settings')
    }
}

/**
 * Checks navigation tabs that are visible based on language
 * 
 * @param {string} lang indicates if user is linked to an employee
 */
export function checkNavTabsLang(lang) {

    cy.visit('/')
    
    if (lang === 'cs') {

        cy.contains('Careers').last().click()

        cy.get('.PriorityNav_Main:first').should('contain', 'Kariéry')
            .and('contain', 'Zaměstnanci')
            .and('contain', 'Šablony')
            .and('contain', 'Knihovna obsahu')
            .and('contain', 'Majetek')
            .and('contain', 'Tým')
            // .and('contain', 'Úkoly')
            // .and('contain', 'Nastavení')

    } else {
        cy.get('.PriorityNav_Main:first').should('contain', 'Careers')
            .and('contain', 'Employees')
            .and('contain', 'Templates')
            .and('contain', 'Content library')
            .and('contain', 'Inventory')
            .and('contain', 'Team')
            // .and('contain', 'Task center')
            // .and('contain', 'Settings')

    }
}

// EMPLOYEE'S ACCOUNT ↓↓↓ ============================================================

/**
 * Checks navigation tabs that are visible based on the fact if the user is linked to an employee
 * 
 * @param {boolean} isLinked indicates if user is linked to an employee
 */
export function checkNavTabs(isLinked) {
    
    if (!isLinked) {
        cy.get('.PriorityNav_Main:first').should('not.contain', 'Career tasks')
            .and('not.contain', 'Team tasks')
            .and('contain', 'My profile')
            .and('not.contain', 'Content library')
            .and('not.contain', 'Team')

    } else {
        cy.get('.PriorityNav_Main:first').should('contain', 'Career tasks')
            .and('contain', 'Team tasks')
            .and('contain', 'My profile')
            .and('contain', 'Content library')
            .and('contain', 'Team')

        cy.get('.PriorityNav_Main:last').should('contain', 'Personal information')
            .and('contain', 'Language')
            .and('contain', 'Change password')

    }
}

// LEGAL ↓↓↓ -------------------------------------------------------------------------

/**
 * Confirms legal content when logged in in employee's account
 */
export function confirmLegalContent() {

    cy.get('button:contains("Confirm legal content!")').click()
    cy.get('div[id^=headlessui-dialog-panel]').should('not.exist')
}

/**
 * Checks legal content when logged in in employee's account, should be filledContent short
 */
export function checkLegalContent() {

    cy.get('div[id^=headlessui-dialog-panel]')
        .should('exist')
        .and('contain', 'Some funky text text.')
        .and('contain', 'That is very important.')
        .and('contain', 'And kind of weird.')
        .and('contain', 'Funkyyy.')
}

// PASSWORD ↓↓↓ -------------------------------------------------------------------------

/**
 * Changes password of the account
 * 
 * @param {string} oldPassword old password
 * @param {string} newPassword new password
 */
export function changePassword(oldPassword, newPassword) {
    cy.get('[name="oldPassword"]').type(oldPassword)
    cy.get('[name="password1"]').type(newPassword)
    cy.get('[name="password2"]').type(newPassword)

    cy.get('button[type="submit"]:contains("Change password")').click()

    cy.url().should('contain', 'app.onbee.link')
}