/**
 * Creates a new template on /templates 
 *
 * @param {string} templateName name of the template
 * @param {string} description description of the template
 */
export function createTemplate(templateName, description) {

    cy.contains('Add').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')

    // fill info
    cy.get('[name="name"]').type(templateName)
    cy.get('[name="description"]').type(description)
    cy.selectOption('Position in geo chart', 'PRAGUE')
    cy.selectOption('Template language', 'en')

    // submit
    cy.get('[type="submit"]').click()

    // check
    cy.url().should('contain', '/templates')
    cy.get('body tr').should('be.visible').and('contain', templateName)
}

/**
 * Opens template of a given name
 *
 * @param {string} templateName name of the template
 */
export function openTemplate(templateName) {

    cy.wait(2500)
    
    // open
    cy.get('table').contains(templateName).click()

    // check
    cy.url().should('contain', '/template-detail/')
}

/**
 * Deletes the template of a given name
 *
 * @param {string} templateName name of the template
 */
 export function deleteTemplate(templateName) {

    cy.delete('Delete template')

    // check
    cy.url().should('contain', '/templates')
    cy.get('body tr').should('be.visible').and('not.contain', templateName)
}

/**
 * Adds milestone of a given name
 *
 * @param {string} milestoneName name of the milestone
 * @param {boolean} inTemplate true when creating in a template, false when creating in employee's career tab 
 */
export function addModule(milestoneName, inTemplate) {

    cy.contains("Add milestone").click()
    cy.selectOption('Career milestone name', milestoneName)

    if (!inTemplate) {
        cy.get('.react-datepicker__input-container').type('01.01.2024')
    }

    // submit
    cy.get('[type="submit"]').click()

    // check
    cy.get('body').should('be.visible').and('contain', milestoneName)
}

/**
 * Opens milestone of a given name
 *
 * @param {string} milestoneName name of the milestone
 */
export function openModule(milestoneName) {

    cy.get('div > div > h4:contains(' + milestoneName + ')').parent().click().should('have.class', 'shadow')
}

/**
 * Deletes milestone of a given name
 *
 * @param {string} milestoneName name of the milestone to be deleted
 */
export function deleteModule(milestoneName) {

    cy.get('div > div > h4:contains(' + milestoneName + ')').parent().children().last().click()

    cy.get('li > div:contains("Delete")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
    cy.get('button:contains("Confirm")').click()

    // check
    cy.get('body').should('be.visible').and('not.contain', milestoneName)
}

/**
 * Adds a section of a given name to the opened milestone
 *
 * @param {string} sectionName name of the section
 */
export function addPhase(sectionName) {

    cy.get('button:contains("Add section")').click()

    // fill info
    cy.get('[name="stadiumName"]').type(sectionName)

    // save
    cy.get('button:contains("Save section")').click()

    // check
    cy.get('body').should('contain', sectionName)
}

/**
 * Deletes section of a given name
 *
 * @param {string} sectionName name of the section
 */
export function deletePhase(sectionName) {

    cy.get('h2:contains("' + sectionName + '")').parent().siblings('div').click()

    cy.get('li > div:contains("Delete")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
    cy.get('button:contains("Confirm")').click()

    // check
    cy.get('body').should('be.visible').and('not.contain', sectionName)
}

/**
 * Adds a task to the last section in the milestone
 *
 * @param {string} taskName name of the task
 * @param {string} description description of the task
 * @param {boolean} inTemplate true when creating in a template, false when creating in employee's career tab 
 * @param {object} content object with content information
 * 
 */
export function addTask(taskName, description, inTemplate, content=null) {
    
    if (inTemplate) {
        cy.get('button:contains("Add task")').last().click()

    } else {
        cy.get('button:contains("Task")').last().click()

    }
    
    // fill info
    cy.get('[name="name"]').type(taskName)
    cy.get('[name="description"]').type(description)
    cy.get('[name="startDue"]').type('1')
    cy.get('[name="reminder"]').type('1')
    cy.get('[name="required"]').check()
    cy.get('[name="duration"]').type('10')

    if( content === null) {
        cy.get('[name="noContentRequired"]').check()

    } else {
        cy.selectOption('Languages', content.lang)
        cy.selectOption('Content type', content.type)
        cy.get('label:contains("Content").required + ').type(content.name +'{enter}')
    
    }

    cy.selectOption('Tags', 'IT')
    cy.selectOption('Tags', 'HR')

    // save
    cy.get('button:contains("Save")').click({force:true})

    // check
    cy.get('.overflow-x-hidden').should('not.exist')
    cy.get('.Toastify__toast-body').should('be.visible')
    cy.get('body').should('be.visible').and('contain', taskName)
}

/**
 * Deletes the task of given name
 *
 * @param {string} taskName name of the task
 */
export function deleteTask(taskName) {

    cy.contains(taskName).parentsUntil('.transition-all').parent().find('button').click()

    cy.get('li > div:contains("Delete task")').click()
    cy.get('div[id^="headlessui-dialog-panel"]').should('be.visible')
    cy.get('button:contains("Confirm")').click()

    // check
    cy.get('body').should('be.visible').and('not.contain', taskName)
}

/**
 * Adds an internal task to the chosen task
 *
 * @param {string} taskName chosen task where the internal task will be added
 * @param {string} description description of the internal task
 * @param {string} internalTaskName name of the internal task
 * @param {object} content content info
 * @param {string} assigneeSurname content info
 * 
 */
export function addInternalTask(taskName, description, internalTaskName, content=null, assigneeSurname='') {

    cy.contains(taskName).click()
    cy.wait(5000)
    cy.get('button:contains("Team tasks")').click()
    cy.get('button:contains("Create new")').click()

    // fill info
    cy.get('[name="name"]').last().type(internalTaskName)
    cy.get('[name="description"]').last().type(description)
    cy.get('[name="startDue"]').last().type('1')
    cy.get('[name="reminder"]').last().type('1')
    // cy.get('label:contains("Assigned to") +').type(assigneeSurname + '{enter}')
    cy.selectOption('Assigned to', assigneeSurname)
    cy.get('[name="viewInternalTaskForOnboardee"]').last().check()
    cy.get('[name="duration"]').last().type('10')

    if( content === null) {
        cy.get('[name="noContentRequired"]').last().check()

    } else {
        cy.selectOption('Languages', content.lang)
        cy.selectOption('Content type', content.type)
        cy.get('label:contains("Content").required + ').last().type(content.name +'{enter}')
    
    }

    cy.selectOption('Tags', 'IT')
    cy.selectOption('Tags', 'HR')

    // save
    cy.get('button:contains("Save")').last().click()

    // check
    cy.contains(taskName).click()
    cy.get('button:contains("Team tasks")').click()
    cy.get('div[id^="headlessui-tabs-panel"]').should('be.visible').and('contain', internalTaskName).last().type('{esc}', { force: true })
}

/**
 * Deletes internal task of the main task with given name
 *
 * @param {string} taskName name of the main task where the internal task will be deleted
 * @param {string} internalTaskName name of the internal task that will be deleted
 */
export function deleteInternalTask(taskName, internalTaskName) {

    cy.contains(taskName).click()
    cy.get('button:contains("Team tasks")').click()
    
    // cy.contains(internalTaskName).parentsUntil('.transition-all').parent().find('svg').last().click({ force: true })
    // cy.get('button:contains("Delete task")').click()

    cy.get('.border-t > .transition-all > .items-end > .px-2').click()
    cy.get('button:contains("Delete task")').click()

    // check
    cy.contains(taskName).click()
    cy.get('button:contains("Team tasks")').click()
    cy.get('div[id^="headlessui-tabs-panel"]').should('be.visible').and('not.contain', internalTaskName).last().type('{esc}', { force: true })

}

/**
 * Renames template
 * 
 * @param {string} oldName template's current name
 * @param {string} newName template's new name
 * @param {string} newDescription template's new description
 */
export function renameTemplate(oldName, newName, newDescription) {

    cy.get('td:contains("' + oldName + '") ++++++ > div').click()
    cy.get('li:contains("Edit")').click()

    // fill info
    cy.get('[name="name"]').clear().type(newName)
    cy.get('[name="description"]').clear().type(newDescription)
    cy.selectOption('Position in geo chart', 'PRAGUE')
    cy.selectOption('Template language', 'en')

    // submit
    cy.get('[type="submit"]').click()

    // check
    cy.get('body tr').should('be.visible').and('contain', newName)
    cy.wait(2500)
}

/**
 * Renames section
 * 
 * @param {string} oldName section's current name
 * @param {string} newName section's new name
 */
export function renamePhase(oldName, newName) {

    cy.get('h2:contains("' + oldName + '")').parent().siblings('div').click()
    cy.get('li > div:contains("Change name")').click()

    // fill info
    cy.get('[name="stadiumName"]').clear().type(newName)

    // save
    cy.get('button:contains("Save section")').click()

    // check
    cy.get('body').should('contain', newName)

}

/**
 * Renames task
 * 
 * @param {string} oldName task's current name
 * @param {string} newName task's new name
 */
export function renameTask(oldName, newName) {

    cy.contains(oldName).parentsUntil('.transition-all').parent().find('button').click()
    cy.get('li > div:contains("Edit task")').click()

    // fill info
    cy.get('[name="name"]').clear().type(newName)

    // save
    cy.get('button:contains("Save")').click()

    // check
    cy.get('.overflow-x-hidden').should('not.exist')
    cy.get('.Toastify__toast-body').should('be.visible')
    cy.get('body').should('be.visible').and('contain', newName)
}


/**
 * Renames internal task
 * 
 * @param {string} mainTaskName main task name
 * @param {string} oldName internal task's current name
 * @param {string} newName internal task's new name
 */
export function renameInternalTask(mainTaskName, oldName, newName) {

    cy.contains(mainTaskName).click()
    cy.get('button:contains("Team tasks")').click()

    cy.get('div:contains("' + oldName + '")').parent().parent().parent().parent().children().last().click()

    // fill info
    cy.get('[name="name"]').last().clear().type(newName)

    // save
    cy.get('button:contains("Save")').last().click({ force: true })

    cy.contains(mainTaskName).click()
    cy.get('button:contains("Team tasks")').click()
    cy.get('div[id^="headlessui-tabs-panel"]').should('be.visible').and('contain', newName).last().type('{esc}', { force: true })
}